import { handleRadioChange } from './helpers';

// Mock DOM methods and properties
document.querySelectorAll = jest.fn();

describe('handleRadioChange', () => {
  let mockRadioGroup;
  let mockRadio;
  let mockElementsToToggle;
  let mockElementsToUpdate;

  beforeEach(() => {
    jest.clearAllMocks();

    mockRadioGroup = {
      dataset: {
        triggerVisibility: 'true',
        isDynamicSetter: 'true',
        targetClass: '.target',
        placeholder: 'old-value'
      },
      querySelector: jest.fn()
    };

    mockRadio = {
      checked: true,
      value: 'test-value',
      closest: jest.fn().mockReturnValue(mockRadioGroup)
    };

    mockElementsToToggle = [
      { 
        getAttribute: jest.fn().mockReturnValue('test-value'),
        classList: { contains: jest.fn().mockReturnValue(false) },
        style: { display: 'none' }
      },
      { 
        getAttribute: jest.fn().mockReturnValue('other-value'),
        classList: { contains: jest.fn().mockReturnValue(false) },
        style: { display: 'block' }
      }
    ];

    mockElementsToUpdate = [
      { tagName: 'INPUT', value: 'old-value is here' },
      { tagName: 'DIV', innerText: 'old-value is there' },
      { tagName: 'SPAN', textContent: 'old-value everywhere' }
    ];
  });

  test('should update visibility and values when radio button is checked', () => {
    document.querySelectorAll.mockReturnValueOnce(mockElementsToToggle).mockReturnValueOnce(mockElementsToUpdate);

    const mockEvent = { target: mockRadio };

    handleRadioChange(mockEvent);

    expect(mockRadio.closest).toHaveBeenCalledWith('.radio-group');
    expect(document.querySelectorAll).toHaveBeenCalledWith('.target');
    expect(mockElementsToToggle[0].style.display).toBe('');
    expect(mockElementsToToggle[1].style.display).toBe('none');
    expect(mockElementsToUpdate[0].value).toBe('test-value is here');
    expect(mockElementsToUpdate[1].innerText).toBe('test-value is there');
    expect(mockElementsToUpdate[2].textContent).toBe('test-value everywhere');
    expect(mockRadioGroup.dataset.placeholder).toBe('test-value');
  });

  test('should not update elements when radio button is unchecked', () => {
    mockRadio.checked = false;
    document.querySelectorAll.mockReturnValueOnce(mockElementsToToggle).mockReturnValueOnce(mockElementsToUpdate);

    const mockEvent = { target: mockRadio };

    handleRadioChange(mockEvent);

    expect(mockRadio.closest).toHaveBeenCalledWith('.radio-group');
    expect(document.querySelectorAll).toHaveBeenCalledWith('.target');
    expect(mockElementsToToggle[0].style.display).toBe('');
    expect(mockElementsToToggle[1].style.display).toBe('none');
    expect(mockElementsToUpdate[0].value).toBe('test-value is here');
    expect(mockElementsToUpdate[1].innerText).toBe('test-value is there');
    expect(mockElementsToUpdate[2].textContent).toBe('test-value everywhere');
  });

  test('should throw an error when querySelectorAll fails', () => {
    document.querySelectorAll.mockImplementation(() => {
      throw new Error('Test error');
    });

    const mockEvent = { target: mockRadio };

    expect(() => {
      handleRadioChange(mockEvent);
    }).toThrow('Test error');
  });
});