/**
 * @jest-environment jsdom
 */

const $ = require('jquery');

// Mock the jQuery Cropper plugin
$.fn.cropper = function(options) {
  return this.each(function() {
    // Store the options on the element
    $(this).data('cropper-options', options);
    
    // Mock cropper methods
    this.cropper = {
      getData: jest.fn(() => ({ width: 100, height: 100, x: 0, y: 0 })),
      getCroppedCanvas: jest.fn(() => ({
        toDataURL: jest.fn(() => 'data:image/png;base64,mockedImageData'),
      })),
      getImageData: jest.fn(() => ({ naturalWidth: 200, naturalHeight: 200 })),
      destroy: jest.fn(),
    };

    // Call the ready callback if it exists
    if (options && options.ready) {
      options.ready.call(this);
    }
  });
};

// Mock the global window object
global.window = {
    URL: {
        createObjectURL: jest.fn(() => 'blob:mocked-url'),
        revokeObjectURL: jest.fn(),
    }
};

global.fetch = jest.fn(() =>
    Promise.resolve({
      json: () => Promise.resolve({ MediaOption: '{"Url":"/path/to/image.jpg"}' }),
    })
);

global.XMLHttpRequest = jest.fn(() => ({
    open: jest.fn(),
    send: jest.fn(),
    setRequestHeader: jest.fn(),
    readyState: 4,
    status: 200,
    responseText: JSON.stringify({ MediaOption: '{"Url":"/path/to/image.jpg"}' })
}));

const QorCropper = require('./qor-cropper.js');

describe('QorCropper', () => {
    let element;
    let qorCropperInstance;

    beforeEach(() => {
        // Set up your HTML fixture for testing
        document.body.innerHTML = `
            <div class="qor-field">
                <div class="qor-field__block qor-file">
                    <div class="qor-fieldset">
                        <textarea class="qor-file__options hidden" data-cropper-title="Crop image" data-cropper-cancel="Cancel" data-cropper-ok="OK" name="QorResource.MediaOption"></textarea>
                        <input class="qor-file__input" type="file" />
                        <span class="qor-file__list">
                            <span class="qor-file__list-item">
                                <span class="qor-file__filename"></span>
                                <span class="qor-cropper__toggle hidden">
                                    <i class="tyk-icon tykon tykon-x qor-cropper__toggle--delete"></i>
                                </span>
                            </span>
                        </span>
                    </div>
                </div>
            </div>
        `;

        element = $('.qor-file__input');

        // Mock fetch for AJAX calls
        global.fetch = jest.fn(() =>
            Promise.resolve({
                json: () => Promise.resolve({ MediaOption: '{"Url":"/path/to/image.jpg"}' }),
            })
        );

        // Initialize QorCropper
        element.qorCropper({
            parent: '.qor-file',
            output: '.qor-file__options',
            list: '.qor-file__list',
            key: 'CropOptions'
        });
    });

    test('should initialize QorCropper correctly', async () => {
        await new Promise(resolve => setTimeout(resolve, 100));

        qorCropperInstance = element.data('qor.cropper');

        expect(qorCropperInstance).toBeDefined();
        expect(qorCropperInstance.$element[0]).toBe(element[0]);
        expect(qorCropperInstance.$parent.length).toBe(1);
        expect(qorCropperInstance.options.key).toBe('CropOptions');
    });

    test('should handle load method', () => {
        // Set up the initial state
        qorCropperInstance.currentFilename = 'test-image.jpg';
        qorCropperInstance.$parent.find('.qor-file__filename').text('');
        qorCropperInstance.$parent.find('.qor-cropper__toggle').first().addClass('hidden');

        // Call the load method
        qorCropperInstance.load('test-url');

        // Check if the filename is updated
        expect(qorCropperInstance.$parent.find('.qor-file__filename').text()).toBe('test-image.jpg');

        // Check if the toggle is no longer hidden
        expect(qorCropperInstance.$parent.find('.qor-cropper__toggle').first().hasClass('hidden')).toBe(false);
    });

    test('should handle delete with click', () => {
        // Set up the initial state
        qorCropperInstance.$list.show();
        expect(qorCropperInstance.$list.css('display')).not.toBe('none');

        // Simulate click on delete button
        const $deleteButton = qorCropperInstance.$parent.find('.qor-cropper__toggle--delete');
        $deleteButton.trigger('click');

        // Check if the list is hidden
        expect(qorCropperInstance.$list.css('display')).toBe('none');

        // Check if the alert is shown
        const $alert = qorCropperInstance.$parent.find('.qor-fieldset__alert');
        expect($alert.length).toBe(1);

        // Check if the data is updated
        expect(qorCropperInstance.data.Delete).toBe(true);

        // Simulate click on undo button
        const $undoButton = $alert.find('.qor-fieldset__undo');
        $undoButton.trigger('click');

        // Check if the list is shown again
        expect(qorCropperInstance.$list.css('display')).not.toBe('none');

        // Check if the alert is removed
        expect(qorCropperInstance.$parent.find('.qor-fieldset__alert').length).toBe(0);

        // Check if the data is updated
        expect(qorCropperInstance.data.Delete).toBeUndefined();
    });
    
    afterEach(() => {
        // Clean up the DOM
        document.body.innerHTML = '';
        jest.restoreAllMocks();
    });
});